;
; DDS generator
;
; SINE and TRIANGLE based on ...
; --------------------------
; Jesper Hansen's <jesperh@telia.com>.miniDDS ( 2000 )
; and a thread at http://www.mikrocontroller.net/topic/244609#new
; many thanks to guest Ulrich
; Resolution on phase, frequency and voltage is improved.
; True 8 Bit and also 9 Bit D/A versions.
; 
; RECTANGLE
; ---------
; Better resolution and less Jitter.
; An optional version with adjustable duty cycle.
;
; SAWTOOTH
; --------
; SLOW and FAST version *
; calculated on the run, no tables used, 9 and 8 Bit
;
; PRECISE version *
; same as sine and triangle (SLOW)

; FSK
; ---
; There is a Frequency Shif Keying option as well ...
; r1,r2 and r3 will hold the other ( Mark ) frequency

; PSK
; ---
; 180 Phase Shift Keying available on sine and triangle wave

; SWEEP
; -----
; allows to inrease frequency at a controlled rate of change
;
; NOISE generation based on ...
; -------------------------
; a PDF from FELJC@LTAM
; (he/she does not mention his/her name)
;
; CONTROL
; -------
; The Mega88 is connected via TWI.
; Loops are interrupted by TWI activities.
; It responds to all TWI addresses, address equals command number.
; Next awaiting a value

; Author = Werner Braun
; Target = ATmega88
; Date   = 2012-03-07
;
; PC6    = RESET
; PC5    = SCL TWI
; PC4    = SDA TWI
; PC3    = out READY 
; PC2    = in A/D analog
; PC1    = 22pF addon
; PC0    = Glitch capacitor on / off

; PB6    = crystal 22 Mc
; PB7    = crystal 22 Mc

; PB2    = OC1B ~ TTL out
; PB0    = in 0 ~ SPACE , 1 ~ MARK for FSK / PSK / CW

; PB1    = D/A Data out 8
; PD0..7 = D/A Data out 0..7
;
;
.include "m88def.inc"
;
; MODE Codes

.equ RectangleFastMode      = 11
.equ SawtoothMode_precise   = 10
.equ RectangleMode_var      = 9
.equ RectangleMode_fixed    = 8
.equ TriangleMode_fast      = 7
.equ TriangleMode_slow      = 6
.equ SineMode_fast          = 5
.equ SineMode_slow          = 4
.equ SawtoothMode_fast      = 3
.equ SawtoothMode_slow      = 2
.equ NoiseMode              = 1
.equ OffMode                = 0

; FSK MODE Codes

.equ SineModeFSK            = 12
.equ TriangleModeFSK        = 13

; PSK MODE Codes

.equ SineModePSK            = 14
.equ TriangleModePSK        = 15

; SWEEP MODE ( sine only )
.equ SweepMode              = 16

;
;
; TWI Slave Receiver Codes
.equ TW_SR_SLA_ACK                  = 0x60;(SLA+W received and ACK returned)													
.equ TW_SR_ARB_LOST_SLA_ACK         = 0x68;(Arbitration lost SLA+R/W as MASTER;  
                                      ; SLA+W received;  ACK returned
.equ TW_SR_GCALL_ACK                = 0x70;(General Call address received;ACK returned									
.equ TW_SR_ARB_LOST_GCALL_ACK       = 0x78;(Arbitration lost SLA+R/W as MASTER;
                                      ;  General Call address received;ACK returned
.equ TW_SR_DATA_ACK                 = 0x80;(prev. adressed (SLA+W) data received;ACK returned					
.equ TW_SR_DATA_NACK                = 0x88;(prev. adressed (SLA+W) data received;ACK NOT returned				
.equ TW_SR_GCALL_DATA_ACK           = 0x90;(prev. adressed (General Call) data received;ACK returned			
.equ TW_SR_GCALL_DATA_NACK          = 0x98;(prev. adressed (General Call) data received;ACK NOT returned	
.equ TW_SR_STOP                     = 0xA0;(STOP received, or repeated START condition)													

; TWI SLAVE configuration
.equ TWI_SLAVE_ADRESSE	= 0x02       ; own SLAVE ADDRESS 1...127 ( does not matter )		
.equ TWI_GENERAL_CALL_enable = 0     ; 1=Generral Call enabled / 0=disabled	

; REGISTER
.def SPEED     = r20
.def DUTY      = r19
.def MODE      = r18
.def ADDRESS   = r17
.def TEMP      = r16
.def DATA      = r15
.def TEMP_SREG = r14
.def LastCopy  = r13              

.def highSpace = r6
.def midSpace  = r5
.def lowSpace  = r4
.def highMark  = r3
.def midMark   = r2
.def lowMark   = r1

.cseg                              ; Start of Code-Segment
.org 0
rjmp RESET                         ; Reset Handler                       

.org TWIaddr
rjmp TWSI                          ; Two-wire Serial Interface Handler 

.org INT_VECTORS_SIZE

RESET:                        
                      
;******************************************************************************
; code
;******************************************************************************


ldi  TEMP,0xFF
out  SPL,TEMP		
ldi  TEMP,0x04
out  SPH,TEMP


; TWI initialization

; TWI TWBR
ldi TEMP, 65
sts TWBR,TEMP
; TWI TWCR control-register
ldi TEMP, 1<<TWINT|1<<TWEA|0<<TWSTA|0<<TWSTO|0<<TWWC|1<<TWEN|1<<TWIE
sts TWCR,TEMP
; TWI TWAR (only SLAVE) ADDRESS-register		
ldi TEMP,(TWI_SLAVE_ADRESSE<<TWA0 | TWI_GENERAL_CALL_ENABLE<<TWGCE)
sts TWAR,TEMP
; respond to all addresses set TWAMR 
ldi TEMP,0xFF
sts TWAMR,TEMP


; start values			
clr DATA
clr ADDRESS
clr MODE
ldi TEMP,0x80
mov DUTY,TEMP
clr LastCopy

; setup adder registers		
clr r4
clr r5
clr r6
clr r24
clr r25	      
clr r26
clr r27
	
Idle:

; clear up STACK
ldi  TEMP,0xFF
out  SPL,TEMP		
ldi  TEMP,0x04
out  SPH,TEMP

; restore adder value
mov r24,r4
mov r25,r5
mov r26,r6

; global enable interrupts
sei			

; PORT initialization

; PUD bit in MCUCR disables the pull-up function 
; for all pins in all ports when set.
;ldi TEMP,0x10
;out MCUCR,TEMP

cbi DDRC,0         ; deglitch is off
cbi DDRC,1         ; addon is off
sbi DDRC,3         ; READY is low
cbi PORTC,3

ser TEMP
out DDRD,TEMP      ; set all PORTD bits as output
sbi DDRB,1         ; Bit 8 is enabled
out PORTD,TEMP     ; FF for start
cbi PORTB,1        ; Bit 8 low
cbi DDRB,0         ; FSK / PSK input

; DEBUG
;ldi MODE,SweepMode
;ldi r24,1
;mov r4,r24
;ldi r25,33
;mov r5,r25
;ldi r26,10
;mov r6,r26

new_game:
sbi PORTC,3        ; READY
cpi  MODE,OffMode  
; wait for a job
breq new_game
cbi PORTC,3        ; BUSY

cpi  MODE,NoiseMode
brne next1
rjmp NoiseLoop
next1:

cpi  MODE,SineMode_slow
brne next2
ldi  r31,high(2*SineTable)
rjmp TableLoop
next2:

cpi  MODE,SineMode_fast
breq CopySineTable
cpi  MODE,SineModeFSK
breq CopySineTable
cpi  MODE,SineModePSK
breq CopySineTable
cpi  MODE,SweepMode
breq CopySineTable

rjmp NoSineTable

; copy long SineTable to SRAM
CopySineTable:

mov  r31,LastCopy
cpi  r31,high(2*SineTable)
breq NoSineTable   ; is copied, no need to repeat

; point to long sine table
ldi  r31,high(2*SineTable)
mov  LastCopy,r31
ldi  r30,low(2*SineTable)
; point to SRAM starting  $3FF +1
; copy every second Byte to SRAM
ldi  YH,0x04
ldi  YL,0x00
sine_to_255:
lpm  r16,z+
ror  r16
andi r16,0x7F
st   -y,r16
lpm  r16,z+
cpi  YL,0x00
brne sine_to_255
ldi  YH,0x03
ldi  YL,0x00
sine_to_510:
lpm  r16,z+
ror  r16
ori  r16,0x80
st   -y,r16
lpm  r16,z+
cpi  YL,0x00
brne sine_to_510
ldi  YH,0x01
ldi  YL,0x00
ldi  r31,0x03
ldi  r30,0x00
sine_to_768:
ld   r16,z+
st   y+,r16
cpi  r31,0x04
brlo sine_to_768
NoSineTable:


cpi  MODE,TriangleMode_fast
breq CopyTriangleTable
cpi  MODE,TriangleModeFSK
breq CopyTriangleTable
cpi  MODE,TriangleModePSK
breq CopyTriangleTable

rjmp NoTriangleTable

; copy long TriangleTable to SRAM
CopyTriangleTable:

mov  r31,LastCopy
cpi  r31,high(2*TriangleTable)
breq NoSineTable   ; is copied, no need to repeat

; point to long Triangle table
ldi  r31,high(2*TriangleTable)
mov  LastCopy,r31
ldi  r30,low(2*TriangleTable)
; point to SRAM starting  $3FF +1
; copy every second Byte to SRAM
ldi  YH,0x04
ldi  YL,0x00
Triangle_to_255:
lpm  r16,z+
ror  r16
andi r16,0x7F
st   -y,r16
lpm  r16,z+
cpi  YL,0x00
brne Triangle_to_255
ldi  YH,0x03
ldi  YL,0x00
Triangle_to_510:
lpm  r16,z+
ror  r16
ori  r16,0x80
st   -y,r16
lpm  r16,z+
cpi  YL,0x00
brne Triangle_to_510
ldi  YH,0x01
ldi  YL,0x00
ldi  r31,0x03
ldi  r30,0x00
Triangle_to_768:
ld   r16,z+
st   y+,r16
cpi  r31,0x04
brlo Triangle_to_768
NoTriangleTable:

cpi  MODE,SineMode_fast
brne next3
rjmp TableLoop_fast
next3:

cpi  MODE,SawtoothMode_fast
brne next4
clr  r27
rjmp ST_fast
next4:

cpi  MODE,SawtoothMode_slow
brne next5
clr  r27
rjmp ST_slow
next5:

cpi  MODE,TriangleMode_slow
brne next6
clr  r27
ldi  r31,high(2*TriangleTable)
rjmp TableLoop
next6:

cpi  MODE,TriangleMode_fast
brne next7
rjmp TableLoop_fast
next7:

cpi  MODE,RectangleMode_fixed
brne next8
rjmp RectangleMode_f
next8:

cpi  MODE,RectangleMode_var
brne next9
rjmp RectangleMode_v
next9:

cpi  MODE,SawtoothMode_precise
brne next10
ldi  r31,high(2*SawtoothTable)
ldi  r30,0xFF
out  PORTD,r30
rjmp TableLoop
next10:

cpi  MODE,RectangleFastMode
brne next11
rjmp RectangleMode_fast
next11:

cpi  MODE,SineModeFSK
brne next12
rjmp TableLoop_FSK
next12:

cpi  MODE,TriangleModeFSK
brne next13

rjmp TableLoop_FSK
next13:

cpi  MODE,TriangleModePSK
brne next14
rjmp TableLoop_PSK
next14:

cpi  MODE,SineModePSK
brne next15
rjmp TableLoop_PSK
next15:

cpi  MODE,SweepMode
brne next16
rjmp SweepLoop
next16:

rjmp Idle           ; NO valid MODE !

TableLoop_FSK:		

  ; 2.2 Mc sampling rate 
  ; 25 Bit resolution on frequency
  ; 360/512 resolution on phase
  ; 8 Bit resolution on voltage
  ; Data tables from $0100 to $03FF in SRAM
  ; adds MARK or SPACE depending on PB0 status
  ; 24 Bit shift max.

    cbi  DDRB,1    ; Bit 8 is off
    cbi  PORTB,1
    cbi  DDRB,0    ; PB0 is input
    cbi  DDRC,0    ; deglitch not needed
    cbi  PORTC,0
    cbi  DDRC,1    ; addon not wanted
    cbi  PORTC,1

    clr  r27
    ser  r30
    ldi  r31,3

    sbi PORTC,3         ; READY

  TL_FSK:               ; cycl.

    sub  r28,r24        ; 1
    sbc  r29,r25        ; 1
    sbc  r30,r26        ; 1
    sbc  r31,r27        ; 1
    ld   r0,z           ; 2
    out  PORTD,r0       ; 1  update BIT 0..7

;   additional for FSK  
    in   r0,PINB        ; 1  get PB0 ...
    bst  r0,0           ; 1  and remember in T flag

    ori  r31,0x02       ; 1  limit to 1.5 * table length
    sub  r28,r24        ; 1
    sbc  r29,r25        ; 1
    sbc  r30,r26        ; 1
    sbc  r31,r27        ; 1   
    ld   r0,z           ; 2  
    sub  r28,r24        ; 1
    sbc  r29,r25        ; 1
    sbc  r30,r26        ; 1
    out  PORTD,r0       ; 1  update BIT 0..7
    sbc  r31,r27        ; 1
    ori  r31,0x02       ; 1  limit to 1.5 * table length
    ld   r0,z           ; 2
    sub  r28,r24        ; 1
    sbc  r29,r25        ; 1
    sbc  r30,r26        ; 1

;   additional for FSK
;   make a decision for MARK or SPACE
    brts TL_Mark        ; 1
    sbc  r31,r27        ; 1
    out  PORTD,r0       ; 1  update BIT 0..7
    mov  r24,lowSpace   ; 1
    mov  r25,midSpace   ; 1
    mov  r26,highSpace  ; 1
    rjmp Waste_two      ; 2  make it 40, we divide by 10 :o(
   TL_Mark:
    out  PORTD,r0       ; 1  update BIT 0..7
    sbc  r31,r27        ; 1
    mov  r24,lowMark    ; 1
    mov  r25,midMark    ; 1
    mov  r26,highMark   ; 1
    ori  r31,0x02       ; 1  limit to 1.5 * table length (not needed here)
    Waste_two:

    rjmp TL_FSK         ; 2 => 40 / 4 = 10 cycles

TableLoop_PSK:		

  ; 2.44 Mc sampling rate 
  ; 25 Bit resolution on frequency
  ; 360/512 resolution on phase
  ; 8 Bit resolution on voltage
  ; Data tables from $0100 to $03FF in SRAM
  ; 180 phase shift depending on change at PB0

    cbi  DDRB,1      ; Bit 8 is off
    cbi  PORTB,1
    cbi  DDRC,0      ; deglitch not needed
    cbi  PORTC,0
    cbi  DDRC,1      ; addon not wanted
    cbi  PORTC,1

    ser  r30
    ldi  r31,3

    ldi  r27,0x01    ; enable PinChangeFlag on PB0
    sts  PCMSK0,r27 
    out  PCIFR,r27   ; clear last toggle

    clr  r27

    sbi PORTC,3     ; READY

    TL_PSK:         ; cycl.
    ld   r0,z       ; 2  
    out  PORTD,r0   ; 1  update BIT 0..7
    sub  r28,r24    ; 1
    sbc  r29,r25    ; 1
    sbc  r30,r26    ; 1
    sbc  r31,r27    ; 1
    ori  r31,0x02   ; 1  limit to 1.5 * table length
    ld   r0,z       ; 2
    sub  r28,r24    ; 1
    out  PORTD,r0   ; 1  update BIT 0..7
    sbc  r29,r25    ; 1
    sbc  r30,r26    ; 1

;   additional for PSK
    in   r20,PCIFR  ; 1  get PB0 toggle
    out  PCIFR,r20  ; 1  clear flag

    sbc  r31,r27    ; 1   
    ld   r0,z       ; 2  
    sub  r28,r24    ; 1
    out  PORTD,r0   ; 1  update BIT 0..7
    sbc  r29,r25    ; 1
    sbc  r30,r26    ; 1
    sbc  r31,r27    ; 1
    ori  r31,0x02   ; 1  limit to 1.5 * table length
    ld   r0,z       ; 2
    sub  r28,r24    ; 1
    sbc  r29,r25    ; 1
    out  PORTD,r0   ; 1  update BIT 0..7
    sbc  r30,r26    ; 1
    sbc  r31,r27    ; 1

;   additional for PSK
;   180 is 256 increments away ( 1 x r31 )
    ori  r31,0x02   ; 1  limit to 1.5 * table length
    sub  r31,r20    ; 1  sub phase

    rjmp TL_PSK     ; 2 => 36 / 4 = 9 cycles

SweepLoop:		

  ; 2.44 Mc sampling rate 
  ; 25 Bit resolution on frequency
  ; 360/512 resolution on phase
  ; 8 Bit resolution on voltage
  ; Data tables from $0100 to $03FF in SRAM
  ; Sweep starts at selected frequency
  ; SPEED controls the rate of change
  ; To control the span, restart or stop it after a certain time...

    cbi  DDRB,1     ; Bit 8 is off
    cbi  PORTB,1
    cbi  DDRC,0     ; deglitch not needed
    cbi  PORTC,0
    cbi  DDRC,1     ; addon not wanted
    cbi  PORTC,1

    RestartSweep:

    clr  r23        ; r23 temporary accumulator for rate of change
    clr  r27        ; r27 empty, used to add carry only

    ; restore adder value
    mov r24,r4
    mov r25,r5
    mov r26,r6
    ; reset accumulatar
    clr  r28
    clr  r29
    ser  r30
    ldi  r31,3

    ser  MODE       ; make sweep restartable at any time ( fake OffMode )

    sbi PORTC,3     ; READY
    cbi PORTC,3     ; place a strobe for trigger
    sbi PORTC,3

    SL:             ; cycl.
    ld   r0,z       ; 2  
    out  PORTD,r0   ; 1  update BIT 0..7
    sub  r28,r24    ; 1
    sbc  r29,r25    ; 1
    sbc  r30,r26    ; 1
    sbc  r31,r27    ; 1
    ori  r31,0x02   ; 1  limit to 1.5 * table length
    ld   r0,z       ; 2
    sub  r28,r24    ; 1
    out  PORTD,r0   ; 1  update BIT 0..7
    sbc  r29,r25    ; 1
    sbc  r30,r26    ; 1
    sbc  r31,r27    ; 1   
    ld   r0,z       ; 2  
    
;   additional to increment frequency
    add  r23,SPEED  ; 1
    adc  r24,r27    ; 1
    adc  r25,r27    ; 1

    out  PORTD,r0   ; 1  update BIT 0..7

;   additional to increment frequency
    adc  r26,r27    ; 1

    sub  r28,r24    ; 1
    sbc  r29,r25    ; 1
    sbc  r30,r26    ; 1
    sbc  r31,r27    ; 1
    ori  r31,0x02   ; 1  limit to 1.5 * table length
    ld   r0,z       ; 2
    out  PORTD,r0   ; 1  update BIT 0..7
    sub  r28,r24    ; 1
    sbc  r29,r25    ; 1
    sbc  r30,r26    ; 1
    sbc  r31,r27    ; 1
    rjmp SL         ; 2 => 36 / 4 = 9 cycles

TableLoop_fast:		

  ; 2.75 Mc sampling rate 
  ; 25 Bit resolution on frequency
  ; 360/512 resolution on phase
  ; 8 Bit resolution on voltage
  ; Data tables from $0100 to $03FF in SRAM

    cbi  DDRB,1     ; Bit 8 is off
    cbi  PORTB,1
    cbi  DDRC,0     ; deglitch not needed
    cbi  PORTC,0
    cbi  DDRC,1     ; addon not wanted
    cbi  PORTC,1

    clr  r27
    ser  r30
    ldi  r31,3

    sbi PORTC,3     ; READY

    TL:             ; cycl.
    ld   r0,z       ; 2  
    out  PORTD,r0   ; 1  update BIT 0..7
    sub  r28,r24    ; 1
    sbc  r29,r25    ; 1
    sbc  r30,r26    ; 1
    sbc  r31,r27    ; 1
    ori  r31,0x02   ; 1  limit to 1.5 * table length
    ld   r0,z       ; 2
    out  PORTD,r0   ; 1  update BIT 0..7
    sub  r28,r24    ; 1
    sbc  r29,r25    ; 1
    sbc  r30,r26    ; 1
    sbc  r31,r27    ; 1   
    ld   r0,z       ; 2  
    sub  r28,r24    ; 1
    out  PORTD,r0   ; 1  update BIT 0..7
    sbc  r29,r25    ; 1
    sbc  r30,r26    ; 1
    sbc  r31,r27    ; 1
    ori  r31,0x02   ; 1  limit to 1.5 * table length
    ld   r0,z       ; 2
    sub  r28,r24    ; 1
    out  PORTD,r0   ; 1  update BIT 0..7
    sbc  r29,r25    ; 1
    sbc  r30,r26    ; 1
    sbc  r31,r27    ; 1
    rjmp TL         ; 2 => 32 / 4 = 8 cycles

TableLoop:		

  ; 2.2 Mc sampling rate 
  ; 26 Bit resolution on frequency
  ; 360/1024 resolution on phase
  ; 9 Bit resolution on voltage
  ; Data tables in FLASH

    sbi  DDRB,1     ; Bit 8 is on
    sbi  DDRC,0     ; deglitch is needed
    cbi  PORTC,0
    sbi  DDRC,1     ; addon is wanted
    cbi  PORTC,1

    clr   r27

    sbi PORTC,3     ; READY

    TL1:            ; cycl.
    lpm  r0,z       ; 3  
    out  PORTD,r0   ; 1  update BIT 0..7
    out  PortB,r31  ; 1  update Bit 8
    add  r28,r24    ; 1
    adc  r29,r25    ; 1
    adc  r30,r26    ; 1
    adc  r31,r27    ; 1
    cbr  r31,4      ; 1  limit to 1.5 * table length
    lpm  r0,z       ; 3
    out  PORTD,r0   ; 1  update BIT 0..7
    out  PortB,r31  ; 1  update Bit 8
    add  r28,r24    ; 1
    adc  r29,r25    ; 1
    adc  r30,r26    ; 1
    adc  r31,r27    ; 1   
    lpm  r0,z       ; 3  
    add  r28,r24    ; 1
    out  PORTD,r0   ; 1  update BIT 0..7
    out  PortB,r31  ; 1  update Bit 8
    adc  r29,r25    ; 1
    adc  r30,r26    ; 1
    adc  r31,r27    ; 1
    cbr  r31,4      ; 1  limit to 1.5 * table length
    lpm  r0,z       ; 3
    add  r28,r24    ; 1
    out  PORTD,r0   ; 1  update BIT 0..7
    out  PortB,r31  ; 1  update Bit 8
    adc  r29,r25    ; 1
    adc  r30,r26    ; 1
    adc  r31,r27    ; 1
    rjmp TL1        ; 2 => 40 / 4 = 10 cycles

SawtoothLoop:

    ST_slow:

  ; 2.75 Mc sampling rate 
  ; 25 Bit resolution on frequency
  ; 360/512 resolution on phase
  ; 9 Bit resolution on voltage

    sbi  DDRB,1      ; Bit 8 is on
    sbi  DDRC,0      ; deglitch is needed
    cbi  PORTC,0
    sbi  DDRC,1      ; addon is wanted
    cbi  PORTC,1

    ldi  r31,1
    clr  r28
    clr  r29
    clr  r30

    sbi PORTC,3      ; READY

    add  r28,r24 	    
    STL1:
    adc  r29,r25     ; 1
    adc  r30,r26     ; 1
    adc  r31,r27     ; 1  
    ori  r31,1       ; 1
    out  PORTD,r30   ; 1  update Bit 0..7
    out  PORTB,r31   ; 1  update Bit 8
    add  r28,r24     ; 1
    adc  r29,r25     ; 1
    adc  r30,r26     ; 1
    adc  r31,r27     ; 1  
    add  r28,r24     ; 1
    ori  r31,1       ; 1
    out  PORTD,r30   ; 1  update Bit 0..7
    out  PORTB,r31   ; 1  update Bit 8
    rjmp STL1;       ; 2  16 / 2 = 8 cycles

    ST_fast:

  ; 4.4 Mc sampling rate 
  ; 24 Bit resolution on frequency
  ; 360/256 resolution on phase
  ; 8 Bit resolution on voltage

    cbi  DDRB,1      ; Bit 8 is off
    cbi  PORTB,1
    cbi  DDRC,0      ; deglitch not needed
    cbi  PORTC,0
    cbi  DDRC,1      ; addon not wanted
    cbi  PORTC,1

    clr  r28
    clr  r29
    clr  r30
	 
    sbi PORTC,3      ; READY

    add  r28,r24 
    STL2:
    adc  r29,r25     ; 1
    adc  r30,r26     ; 1
    out  PORTD,r30   ; 1  update Bit 0..7
    add  r28,r24     ; 1
    adc  r29,r25     ; 1
    adc  r30,r26     ; 1
    add  r28,r24     ; 1
    out  PORTD,r30   ; 1  update Bit 0..7
    rjmp STL2;       ; 2  10 / 2 = 5 cycles

RectangleMode_fast:

  ; 3.666 Mc sampling rate 
  ; 17 Bit resolution on frequency
  ; 360/512 resolution on phase
  ; 1 Bit resolution on voltage

    cbi  DDRB,1      ; Bit 8 is off
    cbi  PORTB,1
    cbi  DDRC,0      ; deglitch not needed
    cbi  PORTC,0
    cbi  DDRC,1      ; addon not wanted
    cbi  PORTC,1
    
    ser r31          ; initial
    out PORTD,r31

    sbi PORTC,3      ; READY

    RM1: 
    add  r28,r24     ; 1
    adc  r29,r25     ; 1
    brcc out11       ; 2
    out  PIND,r31    ;   toggle Bit 0..7
    out11:
    rjmp RM1;        ; 2 6 cycles

RectangleMode_f:

  ; 3.14 Mc sampling rate 
  ; 25 Bit resolution on frequency
  ; 360/512 resolution on phase
  ; 1 Bit resolution on voltage
 
    cbi  DDRB,1      ; Bit 8 is off
    cbi  PORTB,1
    cbi  DDRC,0      ; deglitch not needed
    cbi  PORTC,0
    cbi  DDRC,1      ; addon not wanted
    cbi  PORTC,1
    
    ser r31         ; initial
    out PORTD,r31

    sbi PORTC,3      ; READY
 
    RM2: 
    add  r28,r24     ; 1
    adc  r29,r25     ; 1
    adc  r30,r26     ; 1
    brcc out1        ; 2
    out  PIND,r31    ;   toggle Bit 0..7
    out1:
    rjmp RM2;        ; 2 7 cycles

RectangleMode_v:

  ; 2.444 Mc sampling rate 
  ; 24 Bit resolution on frequency
  ; 360/256 resolution on phase
  ; 1 Bit resolution on voltage
  ; Duty cycle 0 to 255

    cbi  DDRB,1      ; Bit 8 is off
    cbi  PORTB,1
    cbi  DDRC,0      ; deglitch not needed
    cbi  PORTC,0
    cbi  DDRC,1      ; addon not wanted
    cbi  PORTC,1
  
    ser  r31
    clr  r27

    sbi PORTC,3      ; READY

    RM11:            ; on   off
    add  r28,r24     ; 1    1
    adc  r29,r25     ; 1    1
    adc  r30,r26     ; 1    1
    cp   r30,DUTY    ; 1    1
    brsh off         ; 2    1
    nop              ;      1
    out  PORTD,r31   ;      1
    rjmp RM11        ;      2
    off:
    out  PORTD,r27   ; 1
    rjmp RM11        ; 2      9 cycles

NoiseLoop:

;   Linear Feedback Register LFR

    sbi  DDRB,1          ; Bit 8 = PORTB1
    cbi  DDRC,0          ; deglitch not needed
    cbi  PORTC,0
    cbi  DDRC,1          ; addon not wanted
    cbi  PORTC,1
    sbi  PORTC,3         ; READY

    ldi r21,0            ; used for XOR
    ldi r22,0            ; used for XOR
    ldi r28,0x5C         ; starting value
    ldi r29,0xD3
    ldi r30,0xBC
    ldi r31,0xB4
  N0:
    ldi TEMP,0xFF
    ldi R20,32           ; 4*8 = 32 BIT
  N1:
    rol r28              ; rotate 3 bytes over carry
    rol r30
    rol r31
    bst r31,7     
    out PortB,r31        ; have some glitch
    bld r21,0   
    bst r30,5    
    out PORTD,r30
    bld r22,0  
    eor r21,r22   
    bst r28,2     
    bld r22,0    
    eor r21,r22    
    bst r28,0    
    bld r22,0  
    eor r21,r22     
    bst r21,0   
    bld r28,0  
    dec r20              ; finished ?
    brne N1              ; no, go on
    rjmp N0


; TWI INTERRUPPT-Routine		

TWSI:

    in   TEMP_SREG,SREG        ; save SREG 
    lds  TEMP, TWSR            ; get TWI status				
    andi TEMP, 0xF8            ; mask Bit3..7 (prescaler and reserve-BIT)
    cpi  TEMP, TW_SR_SLA_ACK   ; x60	
    breq TWSI_TW_SR_SLA_ACK
    cpi  TEMP, TW_SR_DATA_ACK  ; x80	
    breq TWSI_TW_SR_DATA_ACK    
    cpi  TEMP, TW_SR_STOP      ; xA0	
    breq TWSI_TW_SR_STOP   
; 	
TWI_ERROR:

;   TWI STOP				
    ldi TEMP, (1<<TWINT)|(1<<TWEN)|(1<<TWSTO)
    sts TWCR, TEMP
;   TWI reinit				
    ldi TEMP, (0<<TWEN)
    sts TWCR, TEMP
    ldi TEMP, 65
    sts TWBR,TEMP
    ; TWI TWCR control-register
    ldi TEMP, 1<<TWINT|1<<TWEA|0<<TWSTA|0<<TWSTO|0<<TWWC|1<<TWEN|1<<TWIE
    sts TWCR,TEMP
    ; TWI TWAR (SLAVE only) ADDRESS-register		
    ldi TEMP,(TWI_SLAVE_ADRESSE<<TWA0 | TWI_GENERAL_CALL_ENABLE<<TWGCE)
    sts TWAR,TEMP
	   ; repond to all addresses, set TWAMR 
	   ldi TEMP,0xFF
	   sts TWAMR,TEMP

TWSI_END:	

    ; clear TWINT-Flag 	
    ldi TEMP, 1<<TWINT|1<<TWEA|1<<TWEN|1<<TWIE
    sts TWCR,TEMP
    ; restore SREG 
    out SREG,TEMP_SREG		      
    
    reti

;   SLAVE RECEIVER				

    TWSI_TW_SR_STOP:       ; xA0   10100000 ( STOP or repeated Start )
    lds  ADDRESS,TWDR
    ror  ADDRESS
    andi ADDRESS,0x7F
    rjmp TWSI_END
    TWSI_TW_SR_SLA_ACK:    ; x60   01100000 (SLA+W received-ACK returned)
    lds  ADDRESS,TWDR      ; START next is register number to come in
    ror  ADDRESS
    andi ADDRESS,0x7F
    rjmp TWSI_END
    TWSI_TW_SR_DATA_ACK:   ; x80   10000000 (prev. adressed (SLA+W) data received;  ACK returned	
    lds  DATA,TWDR

    cpi  ADDRESS,0x13      ; DUTY , r19 ?
    brne l19
    mov  DUTY,DATA
    l19:

    cpi ADDRESS,0x12       ; MODE , r18 ?
    brne l24
    cp  MODE,DATA
    breq TWSI_END
    ; MODE has changed
    mov MODE,DATA
    ; clear TWINT-Flag 	
    ldi TEMP, 1<<TWINT|1<<TWEA|1<<TWEN|1<<TWIE
    sts TWCR,TEMP
    rjmp Idle

    l24:
    cpi  ADDRESS,0x18     ; r24 ?
    brne l25
    mov  r24,DATA         ; yes, set
    mov  r4,DATA          ; make a copy in case of FSK or Sweep
    rjmp TWSI_END

    l25:
    cpi  ADDRESS,0x19     ; r25 ?
    brne l26
    mov  r25,DATA         ; yes, set
    mov  r5,DATA          ; make a copy in case of FSK or Sweep
    rjmp TWSI_END

    l26:
    cpi  ADDRESS,0x1a     ; r26 ?
    brne l20
    mov  r26,DATA         ; yes, set
    mov  r6,DATA          ; make a copy in case of FSK or Sweep
    rjmp TWSI_END

    l20:
    cpi ADDRESS,0x14      ; SPEED , r20 ?
    brne l1
    mov SPEED,DATA        ; yes, set
    rjmp TWSI_END
    L1:
    cpi ADDRESS,0x01      ; r1 ?
    brne l2
    mov r1,DATA           ; yes, set
    l2: 
    cpi ADDRESS,0x02      ; r2 ?
    brne l3
    mov r2,DATA           ; yes, set
    l3:
    cpi ADDRESS,0x03      ; r3 ?
    brne l4
    mov r3,DATA           ; yes, set
    l4:
    rjmp TWSI_END


;******************************************************************************
; data tables
;******************************************************************************

; force table to begin at 256 Byte boundary

.org 0x400

SawtoothTable:

.db 0,0,1,1,2,2,3,3,4,4,5,5,6,6,7,7
.db 8,8,9,9,10,10,11,11,12,12,13,13,14,14,15,15
.db 16,16,17,17,18,18,19,19,20,20,21,21,22,22,23,23
.db 24,24,25,25,26,26,27,27,28,28,29,29,30,30,31,31
.db 32,32,33,33,34,34,35,35,36,36,37,37,38,38,39,39
.db 40,40,41,41,42,42,43,43,44,44,45,45,46,46,47,47
.db 48,48,49,49,50,50,51,51,52,52,53,53,54,54,55,55
.db 56,56,57,57,58,58,59,59,60,60,61,61,62,62,63,63
.db 64,64,65,65,66,66,67,67,68,68,69,69,70,70,71,71
.db 72,72,73,73,74,74,75,75,76,76,77,77,78,78,79,79
.db 80,80,81,81,82,82,83,83,84,84,85,85,86,86,87,87
.db 88,88,89,89,90,90,91,91,92,92,93,93,94,94,95,95
.db 96,96,97,97,98,98,99,99,100,100,101,101,102,102,103,103
.db 104,104,105,105,106,106,107,107,108,108,109,109,110,110,111,111
.db 112,112,113,113,114,114,115,115,116,116,117,117,118,118,119,119
.db 120,120,121,121,122,122,123,123,124,124,125,125,126,126,127,127
.db 128,128,129,129,130,130,131,131,132,132,133,133,134,134,135,135
.db 136,136,137,137,138,138,139,139,140,140,141,141,142,142,143,143
.db 144,144,145,145,146,146,147,147,148,148,149,149,150,150,151,151
.db 152,152,153,153,154,154,155,155,156,156,157,157,158,158,159,159
.db 160,160,161,161,162,162,163,163,164,164,165,165,166,166,167,167
.db 168,168,169,169,170,170,171,171,172,172,173,173,174,174,175,175
.db 176,176,177,177,178,178,179,179,180,180,181,181,182,182,183,183
.db 184,184,185,185,186,186,187,187,188,188,189,189,190,190,191,191
.db 192,192,193,193,194,194,195,195,196,196,197,197,198,198,199,199
.db 200,200,201,201,202,202,203,203,204,204,205,205,206,206,207,207
.db 208,208,209,209,210,210,211,211,212,212,213,213,214,214,215,215
.db 216,216,217,217,218,218,219,219,220,220,221,221,222,222,223,223
.db 224,224,225,225,226,226,227,227,228,228,229,229,230,230,231,231
.db 232,232,233,233,234,234,235,235,236,236,237,237,238,238,239,239
.db 240,240,241,241,242,242,243,243,244,244,245,245,246,246,247,247
.db 248,248,249,249,250,250,251,251,252,252,253,253,254,254,255,255

.db 0,0,1,1,2,2,3,3,4,4,5,5,6,6,7,7
.db 8,8,9,9,10,10,11,11,12,12,13,13,14,14,15,15
.db 16,16,17,17,18,18,19,19,20,20,21,21,22,22,23,23
.db 24,24,25,25,26,26,27,27,28,28,29,29,30,30,31,31
.db 32,32,33,33,34,34,35,35,36,36,37,37,38,38,39,39
.db 40,40,41,41,42,42,43,43,44,44,45,45,46,46,47,47
.db 48,48,49,49,50,50,51,51,52,52,53,53,54,54,55,55
.db 56,56,57,57,58,58,59,59,60,60,61,61,62,62,63,63
.db 64,64,65,65,66,66,67,67,68,68,69,69,70,70,71,71
.db 72,72,73,73,74,74,75,75,76,76,77,77,78,78,79,79
.db 80,80,81,81,82,82,83,83,84,84,85,85,86,86,87,87
.db 88,88,89,89,90,90,91,91,92,92,93,93,94,94,95,95
.db 96,96,97,97,98,98,99,99,100,100,101,101,102,102,103,103
.db 104,104,105,105,106,106,107,107,108,108,109,109,110,110,111,111
.db 112,112,113,113,114,114,115,115,116,116,117,117,118,118,119,119
.db 120,120,121,121,122,122,123,123,124,124,125,125,126,126,127,127
.db 128,128,129,129,130,130,131,131,132,132,133,133,134,134,135,135
.db 136,136,137,137,138,138,139,139,140,140,141,141,142,142,143,143
.db 144,144,145,145,146,146,147,147,148,148,149,149,150,150,151,151
.db 152,152,153,153,154,154,155,155,156,156,157,157,158,158,159,159
.db 160,160,161,161,162,162,163,163,164,164,165,165,166,166,167,167
.db 168,168,169,169,170,170,171,171,172,172,173,173,174,174,175,175
.db 176,176,177,177,178,178,179,179,180,180,181,181,182,182,183,183
.db 184,184,185,185,186,186,187,187,188,188,189,189,190,190,191,191
.db 192,192,193,193,194,194,195,195,196,196,197,197,198,198,199,199
.db 200,200,201,201,202,202,203,203,204,204,205,205,206,206,207,207
.db 208,208,209,209,210,210,211,211,212,212,213,213,214,214,215,215
.db 216,216,217,217,218,218,219,219,220,220,221,221,222,222,223,223
.db 224,224,225,225,226,226,227,227,228,228,229,229,230,230,231,231
.db 232,232,233,233,234,234,235,235,236,236,237,237,238,238,239,239
.db 240,240,241,241,242,242,243,243,244,244,245,245,246,246,247,247
.db 248,248,249,249,250,250,251,251,252,252,253,253,254,254,255,255

.db 0,0,1,1,2,2,3,3,4,4,5,5,6,6,7,7
.db 8,8,9,9,10,10,11,11,12,12,13,13,14,14,15,15
.db 16,16,17,17,18,18,19,19,20,20,21,21,22,22,23,23
.db 24,24,25,25,26,26,27,27,28,28,29,29,30,30,31,31
.db 32,32,33,33,34,34,35,35,36,36,37,37,38,38,39,39
.db 40,40,41,41,42,42,43,43,44,44,45,45,46,46,47,47
.db 48,48,49,49,50,50,51,51,52,52,53,53,54,54,55,55
.db 56,56,57,57,58,58,59,59,60,60,61,61,62,62,63,63
.db 64,64,65,65,66,66,67,67,68,68,69,69,70,70,71,71
.db 72,72,73,73,74,74,75,75,76,76,77,77,78,78,79,79
.db 80,80,81,81,82,82,83,83,84,84,85,85,86,86,87,87
.db 88,88,89,89,90,90,91,91,92,92,93,93,94,94,95,95
.db 96,96,97,97,98,98,99,99,100,100,101,101,102,102,103,103
.db 104,104,105,105,106,106,107,107,108,108,109,109,110,110,111,111
.db 112,112,113,113,114,114,115,115,116,116,117,117,118,118,119,119
.db 120,120,121,121,122,122,123,123,124,124,125,125,126,126,127,127
.db 128,128,129,129,130,130,131,131,132,132,133,133,134,134,135,135
.db 136,136,137,137,138,138,139,139,140,140,141,141,142,142,143,143
.db 144,144,145,145,146,146,147,147,148,148,149,149,150,150,151,151
.db 152,152,153,153,154,154,155,155,156,156,157,157,158,158,159,159
.db 160,160,161,161,162,162,163,163,164,164,165,165,166,166,167,167
.db 168,168,169,169,170,170,171,171,172,172,173,173,174,174,175,175
.db 176,176,177,177,178,178,179,179,180,180,181,181,182,182,183,183
.db 184,184,185,185,186,186,187,187,188,188,189,189,190,190,191,191
.db 192,192,193,193,194,194,195,195,196,196,197,197,198,198,199,199
.db 200,200,201,201,202,202,203,203,204,204,205,205,206,206,207,207
.db 208,208,209,209,210,210,211,211,212,212,213,213,214,214,215,215
.db 216,216,217,217,218,218,219,219,220,220,221,221,222,222,223,223
.db 224,224,225,225,226,226,227,227,228,228,229,229,230,230,231,231
.db 232,232,233,233,234,234,235,235,236,236,237,237,238,238,239,239
.db 240,240,241,241,242,242,243,243,244,244,245,245,246,246,247,247
.db 248,248,249,249,250,250,251,251,252,252,253,253,254,254,255,255

.org 0x800

SineTable:
		
.db 255,254,252,251,249,248,246,245,243,241,240,238,237,235,234,232
.db 230,229,227,226,224,223,221,220,218,216,215,213,212,210,209,207
.db 206,204,203,201,200,198,196,195,193,192,190,189,187,186,184,183
.db 181,180,178,177,175,174,172,171,169,168,166,165,164,162,161,159
.db 158,156,155,153,152,151,149,148,146,145,143,142,141,139,138,136
.db 135,134,132,131,130,128,127,125,124,123,121,120,119,117,116,115
.db 114,112,111,110,108,107,106,105,103,102,101,100,98,97,96,95
.db 93,92,91,90,89,87,86,85,84,83,82,80,79,78,77,76
.db 75,74,73,72,70,69,68,67,66,65,64,63,62,61,60,59
.db 58,57,56,55,54,53,52,51,50,49,48,48,47,46,45,44
.db 43,42,41,40,40,39,38,37,36,36,35,34,33,32,32,31
.db 30,29,29,28,27,27,26,25,25,24,23,23,22,21,21,20
.db 19,19,18,18,17,17,16,15,15,14,14,13,13,12,12,11
.db 11,11,10,10,9,9,8,8,8,7,7,7,6,6,6,5
.db 5,5,4,4,4,3,3,3,3,3,2,2,2,2,2,1
.db 1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0
.db 0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1
.db 1,1,2,2,2,2,2,3,3,3,3,3,4,4,4,5
.db 5,5,6,6,6,7,7,7,8,8,8,9,9,10,10,11
.db 11,11,12,12,13,13,14,14,15,15,16,17,17,18,18,19
.db 19,20,21,21,22,23,23,24,25,25,26,27,27,28,29,29
.db 30,31,32,32,33,34,35,36,36,37,38,39,40,40,41,42
.db 43,44,45,46,47,48,48,49,50,51,52,53,54,55,56,57
.db 58,59,60,61,62,63,64,65,66,67,68,69,70,72,73,74
.db 75,76,77,78,79,80,82,83,84,85,86,87,89,90,91,92
.db 93,95,96,97,98,100,101,102,103,105,106,107,108,110,111,112
.db 114,115,116,117,119,120,121,123,124,125,127,128,130,131,132,134
.db 135,136,138,139,141,142,143,145,146,148,149,151,152,153,155,156
.db 158,159,161,162,164,165,166,168,169,171,172,174,175,177,178,180
.db 181,183,184,186,187,189,190,192,193,195,196,198,200,201,203,204
.db 206,207,209,210,212,213,215,216,218,220,221,223,224,226,227,229
.db 230,232,234,235,237,238,240,241,243,245,246,248,249,251,252,254

.db 0,1,3,4,6,7,9,10,12,14,15,17,18,20,21,23
.db 25,26,28,29,31,32,34,35,37,39,40,42,43,45,46,48
.db 49,51,52,54,55,57,59,60,62,63,65,66,68,69,71,72
.db 74,75,77,78,80,81,83,84,86,87,89,90,91,93,94,96
.db 97,99,100,102,103,104,106,107,109,110,112,113,114,116,117,119
.db 120,121,123,124,125,127,128,130,131,132,134,135,136,138,139,140
.db 141,143,144,145,147,148,149,150,152,153,154,155,157,158,159,160
.db 162,163,164,165,166,168,169,170,171,172,173,175,176,177,178,179
.db 180,181,182,183,185,186,187,188,189,190,191,192,193,194,195,196
.db 197,198,199,200,201,202,203,204,205,206,207,207,208,209,210,211
.db 212,213,214,215,215,216,217,218,219,219,220,221,222,223,223,224
.db 225,226,226,227,228,228,229,230,230,231,232,232,233,234,234,235
.db 236,236,237,237,238,238,239,240,240,241,241,242,242,243,243,244
.db 244,244,245,245,246,246,247,247,247,248,248,248,249,249,249,250
.db 250,250,251,251,251,252,252,252,252,252,253,253,253,253,253,254
.db 254,254,254,254,254,254,255,255,255,255,255,255,255,255,255,255
.db 255,255,255,255,255,255,255,255,255,255,255,254,254,254,254,254
.db 254,254,253,253,253,253,253,252,252,252,252,252,251,251,251,250
.db 250,250,249,249,249,248,248,248,247,247,247,246,246,245,245,244
.db 244,244,243,243,242,242,241,241,240,240,239,238,238,237,237,236
.db 236,235,234,234,233,232,232,231,230,230,229,228,228,227,226,226
.db 225,224,223,223,222,221,220,219,219,218,217,216,215,215,214,213
.db 212,211,210,209,208,207,207,206,205,204,203,202,201,200,199,198
.db 197,196,195,194,193,192,191,190,189,188,187,186,185,183,182,181
.db 180,179,178,177,176,175,173,172,171,170,169,168,166,165,164,163
.db 162,160,159,158,157,155,154,153,152,150,149,148,147,145,144,143
.db 141,140,139,138,136,135,134,132,131,130,128,127,125,124,123,121
.db 120,119,117,116,114,113,112,110,109,107,106,104,103,102,100,99
.db 97,96,94,93,91,90,89,87,86,84,83,81,80,78,77,75
.db 74,72,71,69,68,66,65,63,62,60,59,57,55,54,52,51
.db 49,48,46,45,43,42,40,39,37,35,34,32,31,29,28,26
.db 25,23,21,20,18,17,15,14,12,10,9,7,6,4,3,1

.db 255,254,252,251,249,248,246,245,243,241,240,238,237,235,234,232
.db 230,229,227,226,224,223,221,220,218,216,215,213,212,210,209,207
.db 206,204,203,201,200,198,196,195,193,192,190,189,187,186,184,183
.db 181,180,178,177,175,174,172,171,169,168,166,165,164,162,161,159
.db 158,156,155,153,152,151,149,148,146,145,143,142,141,139,138,136
.db 135,134,132,131,130,128,127,125,124,123,121,120,119,117,116,115
.db 114,112,111,110,108,107,106,105,103,102,101,100,98,97,96,95
.db 93,92,91,90,89,87,86,85,84,83,82,80,79,78,77,76
.db 75,74,73,72,70,69,68,67,66,65,64,63,62,61,60,59
.db 58,57,56,55,54,53,52,51,50,49,48,48,47,46,45,44
.db 43,42,41,40,40,39,38,37,36,36,35,34,33,32,32,31
.db 30,29,29,28,27,27,26,25,25,24,23,23,22,21,21,20
.db 19,19,18,18,17,17,16,15,15,14,14,13,13,12,12,11
.db 11,11,10,10,9,9,8,8,8,7,7,7,6,6,6,5
.db 5,5,4,4,4,3,3,3,3,3,2,2,2,2,2,1
.db 1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0
.db 0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1
.db 1,1,2,2,2,2,2,3,3,3,3,3,4,4,4,5
.db 5,5,6,6,6,7,7,7,8,8,8,9,9,10,10,11
.db 11,11,12,12,13,13,14,14,15,15,16,17,17,18,18,19
.db 19,20,21,21,22,23,23,24,25,25,26,27,27,28,29,29
.db 30,31,32,32,33,34,35,36,36,37,38,39,40,40,41,42
.db 43,44,45,46,47,48,48,49,50,51,52,53,54,55,56,57
.db 58,59,60,61,62,63,64,65,66,67,68,69,70,72,73,74
.db 75,76,77,78,79,80,82,83,84,85,86,87,89,90,91,92
.db 93,95,96,97,98,100,101,102,103,105,106,107,108,110,111,112
.db 114,115,116,117,119,120,121,123,124,125,127,128,130,131,132,134
.db 135,136,138,139,141,142,143,145,146,148,149,151,152,153,155,156
.db 158,159,161,162,164,165,166,168,169,171,172,174,175,177,178,180
.db 181,183,184,186,187,189,190,192,193,195,196,198,200,201,203,204
.db 206,207,209,210,212,213,215,216,218,220,221,223,224,226,227,229
.db 230,232,234,235,237,238,240,241,243,245,246,248,249,251,252,254


.org 0xC00

TriangleTable:

.db 255,254,253,252,251,250,249,248,247,246,245,244,243,242,241,240
.db 239,238,237,236,235,234,233,232,231,230,229,228,227,226,225,224
.db 223,222,221,220,219,218,217,216,215,214,213,212,211,210,209,208
.db 207,206,205,204,203,202,201,200,199,198,197,196,195,194,193,192
.db 191,190,189,188,187,186,185,184,183,182,181,180,179,178,177,176
.db 175,174,173,172,171,170,169,168,167,166,165,164,163,162,161,160
.db 159,158,157,156,155,154,153,152,151,150,149,148,147,146,145,144
.db 143,142,141,140,139,138,137,136,135,134,133,132,131,130,129,128
.db 127,126,125,124,123,122,121,120,119,118,117,116,115,114,113,112
.db 111,110,109,108,107,106,105,104,103,102,101,100,99,98,97,96
.db 95,94,93,92,91,90,89,88,87,86,85,84,83,82,81,80
.db 79,78,77,76,75,74,73,72,71,70,69,68,67,66,65,64
.db 63,62,61,60,59,58,57,56,55,54,53,52,51,50,49,48
.db 47,46,45,44,43,42,41,40,39,38,37,36,35,34,33,32
.db 31,30,29,28,27,26,25,24,23,22,21,20,19,18,17,16
.db 15,14,13,12,11,10,9,8,7,6,5,4,3,2,1,0
.db 0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15
.db 16,17,18,19,20,21,22,23,24,25,26,27,28,29,30,31
.db 32,33,34,35,36,37,38,39,40,41,42,43,44,45,46,47
.db 48,49,50,51,52,53,54,55,56,57,58,59,60,61,62,63
.db 64,65,66,67,68,69,70,71,72,73,74,75,76,77,78,79
.db 80,81,82,83,84,85,86,87,88,89,90,91,92,93,94,95
.db 96,97,98,99,100,101,102,103,104,105,106,107,108,109,110,111
.db 112,113,114,115,116,117,118,119,120,121,122,123,124,125,126,127
.db 128,129,130,131,132,133,134,135,136,137,138,139,140,141,142,143
.db 144,145,146,147,148,149,150,151,152,153,154,155,156,157,158,159
.db 160,161,162,163,164,165,166,167,168,169,170,171,172,173,174,175
.db 176,177,178,179,180,181,182,183,184,185,186,187,188,189,190,191
.db 192,193,194,195,196,197,198,199,200,201,202,203,204,205,206,207
.db 208,209,210,211,212,213,214,215,216,217,218,219,220,221,222,223
.db 224,225,226,227,228,229,230,231,232,233,234,235,236,237,238,239
.db 240,241,242,243,244,245,246,247,248,249,250,251,252,253,254,255

.db 0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15
.db 16,17,18,19,20,21,22,23,24,25,26,27,28,29,30,31
.db 32,33,34,35,36,37,38,39,40,41,42,43,44,45,46,47
.db 48,49,50,51,52,53,54,55,56,57,58,59,60,61,62,63
.db 64,65,66,67,68,69,70,71,72,73,74,75,76,77,78,79
.db 80,81,82,83,84,85,86,87,88,89,90,91,92,93,94,95
.db 96,97,98,99,100,101,102,103,104,105,106,107,108,109,110,111
.db 112,113,114,115,116,117,118,119,120,121,122,123,124,125,126,127
.db 128,129,130,131,132,133,134,135,136,137,138,139,140,141,142,143
.db 144,145,146,147,148,149,150,151,152,153,154,155,156,157,158,159
.db 160,161,162,163,164,165,166,167,168,169,170,171,172,173,174,175
.db 176,177,178,179,180,181,182,183,184,185,186,187,188,189,190,191
.db 192,193,194,195,196,197,198,199,200,201,202,203,204,205,206,207
.db 208,209,210,211,212,213,214,215,216,217,218,219,220,221,222,223
.db 224,225,226,227,228,229,230,231,232,233,234,235,236,237,238,239
.db 240,241,242,243,244,245,246,247,248,249,250,251,252,253,254,254
.db 255,254,253,252,251,250,249,248,247,246,245,244,243,242,241,240
.db 239,238,237,236,235,234,233,232,231,230,229,228,227,226,225,224
.db 223,222,221,220,219,218,217,216,215,214,213,212,211,210,209,208
.db 207,206,205,204,203,202,201,200,199,198,197,196,195,194,193,192
.db 191,190,189,188,187,186,185,184,183,182,181,180,179,178,177,176
.db 175,174,173,172,171,170,169,168,167,166,165,164,163,162,161,160
.db 159,158,157,156,155,154,153,152,151,150,149,148,147,146,145,144
.db 143,142,141,140,139,138,137,136,135,134,133,132,131,130,129,128
.db 127,126,125,124,123,122,121,120,119,118,117,116,115,114,113,112
.db 111,110,109,108,107,106,105,104,103,102,101,100,99,98,97,96
.db 95,94,93,92,91,90,89,88,87,86,85,84,83,82,81,80
.db 79,78,77,76,75,74,73,72,71,70,69,68,67,66,65,64
.db 63,62,61,60,59,58,57,56,55,54,53,52,51,50,49,48
.db 47,46,45,44,43,42,41,40,39,38,37,36,35,34,33,32
.db 31,30,29,28,27,26,25,24,23,22,21,20,19,18,17,16
.db 15,14,13,12,11,10,9,8,7,6,5,4,3,2,1,0

.db 255,254,253,252,251,250,249,248,247,246,245,244,243,242,241,240
.db 239,238,237,236,235,234,233,232,231,230,229,228,227,226,225,224
.db 223,222,221,220,219,218,217,216,215,214,213,212,211,210,209,208
.db 207,206,205,204,203,202,201,200,199,198,197,196,195,194,193,192
.db 191,190,189,188,187,186,185,184,183,182,181,180,179,178,177,176
.db 175,174,173,172,171,170,169,168,167,166,165,164,163,162,161,160
.db 159,158,157,156,155,154,153,152,151,150,149,148,147,146,145,144
.db 143,142,141,140,139,138,137,136,135,134,133,132,131,130,129,128
.db 127,126,125,124,123,122,121,120,119,118,117,116,115,114,113,112
.db 111,110,109,108,107,106,105,104,103,102,101,100,99,98,97,96
.db 95,94,93,92,91,90,89,88,87,86,85,84,83,82,81,80
.db 79,78,77,76,75,74,73,72,71,70,69,68,67,66,65,64
.db 63,62,61,60,59,58,57,56,55,54,53,52,51,50,49,48
.db 47,46,45,44,43,42,41,40,39,38,37,36,35,34,33,32
.db 31,30,29,28,27,26,25,24,23,22,21,20,19,18,17,16
.db 15,14,13,12,11,10,9,8,7,6,5,4,3,2,1,0
.db 0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15
.db 16,17,18,19,20,21,22,23,24,25,26,27,28,29,30,31
.db 32,33,34,35,36,37,38,39,40,41,42,43,44,45,46,47
.db 48,49,50,51,52,53,54,55,56,57,58,59,60,61,62,63
.db 64,65,66,67,68,69,70,71,72,73,74,75,76,77,78,79
.db 80,81,82,83,84,85,86,87,88,89,90,91,92,93,94,95
.db 96,97,98,99,100,101,102,103,104,105,106,107,108,109,110,111
.db 112,113,114,115,116,117,118,119,120,121,122,123,124,125,126,127
.db 128,129,130,131,132,133,134,135,136,137,138,139,140,141,142,143
.db 144,145,146,147,148,149,150,151,152,153,154,155,156,157,158,159
.db 160,161,162,163,164,165,166,167,168,169,170,171,172,173,174,175
.db 176,177,178,179,180,181,182,183,184,185,186,187,188,189,190,191
.db 192,193,194,195,196,197,198,199,200,201,202,203,204,205,206,207
.db 208,209,210,211,212,213,214,215,216,217,218,219,220,221,222,223
.db 224,225,226,227,228,229,230,231,232,233,234,235,236,237,238,239
.db 240,241,242,243,244,245,246,247,248,249,250,251,252,253,254,255










